﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ModelFileManager
{
    static class Common
    {
        public const int MAX_MODELS = 255;

        // ----------------------------------------------------------------------------------------------------------------------

        public static string GetModelName(string modelfile)
        {
            string result = "";

            if (File.Exists(modelfile))
            {
                result = "NONE";
                using (StreamReader sr = new StreamReader(modelfile))
                {
                    string nextline = sr.ReadLine();
                    if (nextline.ToUpper().StartsWith("NAME"))
                        result = nextline.Substring(nextline.IndexOf("=") + 1);
                }
            }

            return result;
        }

        // ----------------------------------------------------------------------------------------------------------------------

        public static void SetModelName(string modelFile, string newName)
        {
            string tempFile = modelFile.Replace(".ini", ".new");

            if (File.Exists(modelFile))
            {
                // open a reader to read the original file
                using (StreamReader sr = new StreamReader(modelFile))
                {
                    // open a writer to save the new file content
                    using (StreamWriter sw = new StreamWriter(tempFile))
                    {
                        // scan the whole file
                        while (sr.Peek() > -1)
                        {
                            // replace the name line
                            string nextLine = sr.ReadLine();
                            if (nextLine.ToUpper().StartsWith("NAME"))
                                sw.Write("name=" + newName + "\n");
                            else
                                sw.Write(nextLine + "\n");
                        }
                    }
                }

                // remove the original file
                File.Delete(modelFile);
                // copy in the new file
                File.Move(tempFile, modelFile);
            }
                
        }

        // ----------------------------------------------------------------------------------------------------------------------

        public static void WriteError(string location, Exception ex)
        {
            string ErrorFile = Path.Combine(Application.StartupPath, "errors.txt");
            using (StreamWriter sw = new StreamWriter(ErrorFile, false))
            {
                sw.WriteLine("Error occurred " + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString());
                sw.WriteLine("---------------------------------------------------------------------------------------------");
                sw.WriteLine("Location:  " + location);
                sw.WriteLine("Exception: " + ex.Message + ex.StackTrace);
                sw.WriteLine("---------------------------------------------------------------------------------------------");
            }
        }

    }
}
