﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ModelFileManager
{
    public partial class Restore : Form
    {
        public string BackupDirectory = "";
        public string ModelDirectory = "";

        // ----------------------------------------------------------------------------------------------------------------------

        public Restore()
        {
            InitializeComponent();
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void Restore_Activated(object sender, EventArgs e)
        {
            // only do once
            if (cboBackups.Items.Count == 0)
            {
                // the backups are named by date
                string[] subdirectoryEntries = Directory.GetDirectories(BackupDirectory);
                foreach (string subdirectory in subdirectoryEntries)
                {
                    string d = subdirectory.Substring(BackupDirectory.Length + 1);
                    string backupname = String.Format("{0}-{1}-{2} {3}:{4}:{5}", d.Substring(0, 4), d.Substring(4, 2), d.Substring(6, 2), d.Substring(9, 2), d.Substring(11, 2), d.Substring(13, 2));
                    cboBackups.Items.Add(backupname);
                }

                // init Model list (wait until a backup set is selected to fill)
                cboModels.Items.Clear();
                cboModels.Items.Add("ALL");
                cboModels.SelectedIndex = -1;
                cboModels.Enabled = false;

                // init Destination list
                cboDestination.Items.Clear();
                for (int i = 1; i <= Common.MAX_MODELS; i++)
                {
                    string modelFile = Path.Combine(ModelDirectory, String.Format("model{0}.ini", i));
                    if (File.Exists(modelFile))
                    {
                        string modelName = Common.GetModelName(modelFile);
                        cboDestination.Items.Add(i.ToString() + " - " + modelName);
                    }
                }
                cboDestination.Enabled = false;
                btnEdit.Enabled = false;

                // init Diff funtion
                btnDiff.Enabled = false;
            }
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnCancel_Click(object sender, EventArgs e)
        {
            // the DialogResult only controls whether we refresh or not
            this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.Close();
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnRestore_Click(object sender, EventArgs e)
        {
            try
            {
                // Check that a particular backup was selected
                if (cboBackups.SelectedIndex >= 0)
                {
                    if (cboModels.SelectedIndex == -1)
                    {
                        MessageBox.Show("Select a model to restore.", "Sorry", MessageBoxButtons.OK, MessageBoxIcon.Question);
                    }
                    else if (cboModels.SelectedIndex == 0)
                    {

                        // Confirm with the user
                        if (MessageBox.Show("Are you SURE you want to restore ALL MODELS from the \n\n\t[" + cboBackups.Text + "]\n\nbackup?", "Confirm", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation) == System.Windows.Forms.DialogResult.Yes)
                        {
                            Cursor.Current = Cursors.WaitCursor;

                            // formulate the source and destination of the restore
                            string BackupName = cboBackups.Text.Substring(0, 4) + cboBackups.Text.Substring(5, 2) + cboBackups.Text.Substring(8, 2) + "-" + cboBackups.Text.Substring(11, 2) + cboBackups.Text.Substring(14, 2) + cboBackups.Text.Substring(17, 2);
                            string srcPath = Path.Combine(BackupDirectory, BackupName);
                            string dstPath = ModelDirectory;

                            // continue if both paths exist
                            if (Directory.Exists(srcPath) && Directory.Exists(dstPath))
                            {
                                // delete all the files from the destination
                                string[] killfiles = Directory.GetFiles(dstPath);
                                foreach (string killfile in killfiles)
                                    File.Delete(killfile);

                                // copy all the new files in
                                string[] newfiles = Directory.GetFiles(srcPath);
                                foreach (string newfile in newfiles)
                                    File.Copy(newfile, Path.Combine(dstPath, Path.GetFileName(newfile)));
                            }

                            // Acknowledge
                            Cursor.Current = Cursors.Default;
                            MessageBox.Show("Restore Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Information);

                            // cause the main list to refresh
                            this.DialogResult = System.Windows.Forms.DialogResult.OK;

                            this.Close();
                        }

                    }
                    else
                    {

                        if (cboDestination.SelectedIndex >= 0)
                        {
                            string srcModelItem = cboModels.Text;
                            string modelName = srcModelItem.Substring(srcModelItem.IndexOf("-") + 1).Trim();
                            int srcModelNo = Convert.ToInt16(srcModelItem.Substring(0, srcModelItem.IndexOf("-") - 1));

                            string dstModel = cboDestination.Text;
                            int dstModelNo = Convert.ToInt16(dstModel.Substring(0, dstModel.IndexOf("-") - 1));

                            string BackupName = cboBackups.Text.Substring(0, 4) + cboBackups.Text.Substring(5, 2) + cboBackups.Text.Substring(8, 2) + "-" + cboBackups.Text.Substring(11, 2) + cboBackups.Text.Substring(14, 2) + cboBackups.Text.Substring(17, 2);
                            string srcFileName = Path.Combine(BackupDirectory, BackupName, "model" + srcModelNo.ToString() + ".ini");
                            string dstFileName = Path.Combine(ModelDirectory, "model" + dstModelNo.ToString() + ".ini");

                            if (File.Exists(srcFileName))
                            {
                                // Confirm with the user
                                if (MessageBox.Show("Are you SURE you want to restore model \n\n[" + modelName + "] from [" + cboBackups.Text + "]\n\nover model slot #" + dstModelNo.ToString() + "?", "Confirm", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation) == System.Windows.Forms.DialogResult.Yes)
                                {
                                    // Do it
                                    Cursor.Current = Cursors.WaitCursor;
                                    if (File.Exists(dstFileName))
                                        File.Delete(dstFileName);
                                    File.Copy(srcFileName, dstFileName);
                                    Cursor.Current = Cursors.Default;

                                    // Acknowledge
                                    MessageBox.Show("Restore Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Information);

                                    // cause the main list to refresh
                                    this.DialogResult = System.Windows.Forms.DialogResult.OK;

                                    this.Close();
                                }
                            }
                            else
                                throw new Exception("Unexpected Error during restore: Unable to find source file.");

                        }
                        else
                            MessageBox.Show("Select a model slot to restore to.", "Sorry", MessageBoxButtons.OK, MessageBoxIcon.Question);
                    }

                }
                else
                    MessageBox.Show("Select a backup date to restore.", "Sorry", MessageBoxButtons.OK, MessageBoxIcon.Question);
            }
            catch (Exception ex)
            {
                Cursor.Current = Cursors.Default;
                MessageBox.Show("Operation Was NOT Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Common.WriteError("btnRestore_Click()", ex);
            }
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void cboBackups_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cboBackups.SelectedIndex >= 0)
            {
                // refill and disable the model dropdown list
                cboModels.Items.Clear();
                cboModels.Items.Add("ALL");

                string theDate = cboBackups.Items[cboBackups.SelectedIndex].ToString();
                string dateDirectory = theDate.Substring(0, 4) + theDate.Substring(5, 2) + theDate.Substring(8, 2) + "-" + theDate.Substring(11, 2) + theDate.Substring(14, 2) + theDate.Substring(17, 2);
                string restorePath = Path.Combine(BackupDirectory, dateDirectory);

                for (int i = 1; i <= Common.MAX_MODELS; i++)
                {
                    string modelFile = Path.Combine(restorePath, String.Format("model{0}.ini", i));
                    if (File.Exists(modelFile))
                    {
                        string modelName = Common.GetModelName(modelFile);
                        if (modelName != "NONE")
                            cboModels.Items.Add(i.ToString() + " - " + modelName);
                    }
                }
                cboModels.Enabled = true;
                cboModels.Text = "";
                cboModels.SelectedIndex = -1;
                btnEdit.Enabled = false;

                // disable the destination dropdown list
                cboDestination.SelectedIndex = -1;
                cboDestination.Enabled = false;
                btnDiff.Enabled = false;
            }
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void cboModels_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cboModels.SelectedIndex >= 0)
            {
                if (cboModels.SelectedIndex == 0) // all selected
                {
                    cboDestination.SelectedIndex = -1;
                    cboDestination.Text = "";
                    cboDestination.Enabled = false;
                    btnEdit.Enabled = false;
                    btnDiff.Enabled = false;
                }
                else
                {
                    cboDestination.Enabled = true;
                    btnEdit.Enabled = true;
                }
            }
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void cboDestination_SelectedIndexChanged(object sender, EventArgs e)
        {
            btnDiff.Enabled = (cboDestination.SelectedIndex >= 0);
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnDiff_Click(object sender, EventArgs e)
        {
            if (cboDestination.SelectedIndex >= 0)
            {
                string srcModelItem = cboModels.Text;
                string modelName = srcModelItem.Substring(srcModelItem.IndexOf("-") + 1).Trim();
                int srcModelNo = Convert.ToInt16(srcModelItem.Substring(0, srcModelItem.IndexOf("-") - 1));

                string dstModel = cboDestination.Text;
                int dstModelNo = Convert.ToInt16(dstModel.Substring(0, dstModel.IndexOf("-") - 1));

                string BackupName = cboBackups.Text.Substring(0, 4) + cboBackups.Text.Substring(5, 2) + cboBackups.Text.Substring(8, 2) + "-" + cboBackups.Text.Substring(11, 2) + cboBackups.Text.Substring(14, 2) + cboBackups.Text.Substring(17, 2);
                string srcFileName = Path.Combine(BackupDirectory, BackupName, "model" + srcModelNo.ToString() + ".ini");
                string dstFileName = Path.Combine(ModelDirectory, "model" + dstModelNo.ToString() + ".ini");

                if (File.Exists(srcFileName) && File.Exists(dstFileName))
                {
                    try
                    {

                        string exeFile = ConfigurationManager.AppSettings["DiffViewer"];
                        if ((exeFile == "") || (!File.Exists(exeFile)))
                        {
                            MessageBox.Show("Please set the [DiffViewer] setting in the ModelFileManager.exe.config file.", "Sorry", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                        else
                        {                            
                            string argTemplate = ConfigurationManager.AppSettings["DiffViewerArgs"];
                            if (argTemplate == "")
                                argTemplate = "{0} {1}";
                            string args = String.Format(argTemplate, "\"" + srcFileName + "\"", "\"" + dstFileName + "\"");

                            ProcessStartInfo startInfo = new ProcessStartInfo();
                            startInfo.FileName = exeFile;
                            startInfo.Arguments = args;
                            Process.Start(startInfo);
                        }

                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Operation Was NOT Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        Common.WriteError("btnDiff_Click()", ex);
                    }
                }
            }
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnEdit_Click(object sender, EventArgs e)
        {
            if (cboModels.SelectedIndex >= 1)   // skip the first
            {
                string srcModelItem = cboModels.Text;
                string modelName = srcModelItem.Substring(srcModelItem.IndexOf("-") + 1).Trim();
                int srcModelNo = Convert.ToInt16(srcModelItem.Substring(0, srcModelItem.IndexOf("-") - 1));

                string BackupName = cboBackups.Text.Substring(0, 4) + cboBackups.Text.Substring(5, 2) + cboBackups.Text.Substring(8, 2) + "-" + cboBackups.Text.Substring(11, 2) + cboBackups.Text.Substring(14, 2) + cboBackups.Text.Substring(17, 2);
                string srcFileName = Path.Combine(BackupDirectory, BackupName, "model" + srcModelNo.ToString() + ".ini");

                if (File.Exists(srcFileName))
                {
                    string exeFile = ConfigurationManager.AppSettings["BackupViewer"];
                    if ((exeFile == "") || (!File.Exists(exeFile)))
                    {
                        Viewer viewer = new Viewer(srcFileName);
                        viewer.ShowDialog();
                    }
                    else
                    {
                        string argTemplate = ConfigurationManager.AppSettings["BackupViewerArgs"];
                        if (argTemplate == "")
                            argTemplate = "{0}";
                        string args = String.Format(argTemplate, "\"" + srcFileName + "\"");

                        ProcessStartInfo startInfo = new ProcessStartInfo();
                        startInfo.FileName = exeFile;
                        startInfo.Arguments = args;
                        Process.Start(startInfo);
                    }
                }
            }
            else
                MessageBox.Show("Select a model from the backup set.", "Sorry", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

    }
}
